﻿CompilerIf #PB_Compiler_Unicode
	#XmlEncoding = #PB_UTF8
CompilerElse 
	#XmlEncoding = #PB_Ascii
CompilerEndIf

DeclareModule Dialog
	
	;> ===== KONSTANTEN				; diverse Konstanten, z.B. #align...,  #Expand...
		
		#alignTop		= 1								; Konstanten für das Alignment von Elementen
		#alignBottom	= 2
		#alignLeft		= 4
		#alignRight		= 8
		#alignCenter	= 16
		
		#alignTopLeft			= #alignTop | #alignLeft
		#alignTopCenter		= #alignTop | #alignCenter
		#alignTopRight			= #alignTop | #alignRight
		#alignCenterLeft		= #alignCenter | #alignLeft
		#alignCenterRight		= #alignCenter | #alignRight
		#alignLeftCenter		= #alignCenterLeft
		#alignRightCenter		= #alignCenterRight
		#alignBottomLeft		= #alignBottom | #alignLeft
		#alignBottomCenter	= #alignBottom | #alignCenter
		#alignBottomRight		= #alignBottom | #alignRight
		
		#NodeErrorMessage_None	= 0					; Konstanten für die Art der Anzeige von Fehlermeldungen
		#NodeErrorMessage_Short	= 1
		#NodeErrorMessage_Long	= 2
		
		#Expand_Yes				= -2						; Expand-Konstanten für die diversen Layout-Elemente (wie z.B. GridBox, SingleBox, ...)
		#Expand_No				= -3			
		#Expand_Equal			= -4
		#Expand_Vertical   	= -5
		#Expand_Horizontal 	= -6
		
		#XMLout_LineBreak			= 1					; Attribut-Konstanten für:  SetXMLOutputFormat(Attribut, Value)
		#XMLout_Indent				= 2
		#XMLout_UseRealTab		= 3
		#XMLout_AlignLineBreak	= 4
		
		#XmlDefaultMargin$ = "10"
		#XmlDefaultSpacing = 5
		;<
	
	Enumeration	XML_NodeID
		#Node_FirstNode
		
		#Node_Dialog = #Node_FirstNode
		#Node_Window
		#Node_HBox
		#Node_VBox
		#Node_Gridbox
		#Node_Multibox
		#Node_Singlebox
		
		#Node_Button
		#Node_Buttonimage
		#Node_Calendar
		#Node_Canvas
		#Node_Checkbox
		#Node_Combobox
		#Node_Container
		#Node_Date
		#Node_Editor
		#Node_Explorercombo
		#Node_Explorerlist
		#Node_Explorertree
		#Node_Frame
		#Node_Hyperlink
		#Node_IPaddress
		#Node_Image
		#Node_Listicon
		#Node_Listview
		#Node_Option
		#Node_Panel
		#Node_Progressbar
		#Node_Scrollarea
		#Node_Scrollbar
		#Node_Spin
		#Node_Splitter
		#Node_String
		#Node_Text
		#Node_Trackbar
		#Node_Tree
		#Node_Web
		#Node_Scintilla
		
		#Node_Tab
		#Node_Empty
		
		#Node_LastNode = #Node_Empty
	EndEnumeration
	
	;> ===== Diverse Prozeduren, u.A. zum Bearbeiten des XML-Textes
		
		Declare		ShowNodeErrors(State)							; zum de-/aktiviert von Nachrichten, die bei Fehlern von Close-Nodes angezeigt werden
		Declare 		ClearXML(ClearNodeList=#True)					; löschen bzw. zurücksetzen des bisher erzeugten XML-Codes
		Declare.s	GetXML()												; abrufen des bisher erzeugten XML-Codes
		Declare		SetXML(XML_Text$, XmlConvert=#False)		; setzt den XML-Code. alle folgenden Befehle fügen ihren code hier an
		Declare.s	AddXML(XML_Text$, XmlConvert=#False)		; zum einfügen diverser eigener XML-Codes
		Declare		SetXMLOutputFormat(Attribut, Value)			; hiermit können Parameter für die Formatierung des erzeugten XML-Codes gesetzt werden (z.B. ob ein #CR angefüft wird oder ob und wie weit Zeilen eingerückt werden)
		Declare.s	T2H(Text$)
		Declare.s	CloseNode(NodeType = #PB_Any)					; statt immer den passenden Endxxx()-Befehl zu nehmen, kann auch einfach CloseNode() genommen werden. es wird automatisch das zuletzt geöffnete Node geschlossen.
																				;<
	
		;> ===== XML-Layout Elemente
			
			Declare.s	Dialogs()
			Declare.s 	Window(ID, Name$, Titel$, Flags=0, Width=#PB_Ignore, Height=#PB_Ignore, Param$="")
			Declare.s	vBox(Expand=#PB_Default, Align=#PB_Default, Spacing=#PB_Default, XmlParam$="")
			Declare.s	hBox(Expand=#PB_Default, Align=#PB_Default, Spacing=#PB_Default, XmlParam$="")
			Declare.s	GridBox(Columns=#PB_Default, ColSpacing=#PB_Default, RowSpacing=#PB_Default, ColExpand=#PB_Default, RowExpand=#PB_Default, Param$="")
			Declare.s	MultiBox(Expand=#PB_Default, Align=#PB_Default, Margin$="0", XmlParam$="")
			Declare.s	SingleBox(Expand=#PB_Default, Align=#PB_Default, Margin$="0", Expandwidth=0, Expandheight=0, XmlParam$="")
			Declare.s	Empty(Width=#PB_Default, Height=#PB_Default, XmlParam$="")
		;<
		;> ===== XML-Layout Elemente					(Close-Nodes)
			
			Declare.s	EndDialogs()
			Declare.s	EndWindow()											; End- bzw Close-Node für einen vorherigen Window()-Container
			Declare.s	EndVBox()
			Declare.s	EndHBox()
			Declare.s	EndGridBox()										; End- bzw Close-Node für einen vorherigen GridBox()-Container
			Declare.s	EndMultiBox()
			Declare.s	EndSingleBox()
			;<
		
	;> ===== PuraBasic Container-Elemente
		
		Declare.s	Container(ID=#PB_Ignore, Name$="", Flags=0, Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
		Declare.s	Frame(ID=#PB_Ignore, Name$="", Text$="", Flags=0, Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
		Declare.s	Panel(ID=#PB_Ignore, Name$="", Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
		Declare.s	ScrollArea(ID=#PB_Ignore, Name$="", ScrollAreaWidth=#PB_Default, ScrollAreaHeight=#PB_Default, ScrollStep=#PB_Default, Flags=0, Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
		Declare.s	Splitter(ID, Name$, Gadget1, Gadget2, Flags=0, Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
		;<
		;> ===== PuraBasic Container-Elemente		(Close-Nodes)
		
		Declare.s	EndContainer()
		Declare.s	EndFrame()
		Declare.s	EndPanel()
		Declare.s	EndScrollArea()
		Declare.s	EndSplitter()
		;<
		
		;> ===== PuraBasic Gadgets				TODO: Es müssen noch diverse weitere PB-Gadgets gewrapped werden
		
		Declare.s	Button(ID=#PB_Ignore, Name$="", Text$="", Flags=0, Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
		Declare.s	ButtonImage(ID=#PB_Ignore, Name$="", ImageID=0 , Flags=0, Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
		Declare.s	Calendar(ID=#PB_Ignore, Name$="", Date=#PB_Default , Flags=0, Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
		Declare.s	Canvas(ID=#PB_Ignore, Name$="", Flags=0, Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
		Declare.s	CheckBox(ID=#PB_Ignore, Name$="", Text$="", Flags=0, Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
		Declare.s	ComboBox(ID=#PB_Ignore, Name$="", Flags=0, Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
		
		Declare.s	Image(ID=#PB_Ignore, Name$="", ImageID=#PB_Ignore , Flags=0, Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
		
		Declare.s	Text(ID=#PB_Ignore, Name$="", Text$="", Flags=0, Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
		;<
	
	Macro EndNode(NodeType = #PB_Any)							; Macro, um statt CloseNode() auch EndNode() nutzen zu können
		Dialog::CloseNode(NodeType)
	EndMacro
	
EndDeclareModule
Module Dialog
	
	;> -----  VARIABLEN   -  Globale Variablen und #Konstanten   (innerhalb dieses Moduls)
		
		Global	XMLBuffer$												; Variable, die den gesamten, bisher erzeugten XML-Code beinhaltet
		Global	LastXMLLine$											; die zu letzt erzeugte XML-Zeile sowie alle Close-Nodes  =>  wir nur für Error-Meldungen verwendet
		Global	NewList NodeTree.l()									; Liste aller offenen Nodes. Beim Schließen einer Node wird der letzte Eintrag gelöscht.
		Global	ShowNodeErrors.l = #NodeErrorMessage_Short	; should be changed to #NodeErrorMessage_None on life-projects
		
		Global	XMLout_LineBreak			= #True					; erstellt beim erzeugten XML-Code nach jeder Zeile einen Zeilenumbruch
		Global	XMLout_Indent				= #False					; legt fest, um wieviele Zeichen der Text bei einer Sub-Node eingerückt wird
		Global	XMLout_UseRealTab			= #False					; nutzt zur einrücken des Textes TABs (Chr(9)), statt Leerzeichen.
		Global	XMLout_AlignLineBreak	= #False					; Durch den automatischen Align-Befehl in allen 'Gadgets' werden evtl. automatisch SingleBoxen-Container erstellt.
																				; mit XMLout_AlignLineBreak kann festgelegt werden, ob auch hierfür Zeilenumbruch und Einrücken genutzt wird.
		
		Global	CR$ = #CR$												; Je nach Status von XMLout_LineBreak beinhaltet CR$ ein #CR$ oder nicht.
		
		#Expand_No$				= "'no'"
		#Expand_Yes$			= "'yes'"
		#Expand_Equal$			= "'equal'"
		#Expand_Vertical$   	= "'vertical'"
		#Expand_Horizontal$ 	= "'horizontal'"
		#Expand_Item$			= "'item:"
		
		#Node_FirstNode	= #Node_Dialog
		#Node_LastNode		= #Node_Empty
		;<
		;> -----  VARIABLEN   -  Definition der Strings für </Close-Nodes>
		
		Global	Dim	NodeType$(#Node_LastNode)
		
		NodeType$(#Node_Dialog) = "</dialogs>"
		NodeType$(#Node_Window) = "</window>"
		NodeType$(#Node_HBox) = "</hbox>"
		NodeType$(#Node_VBox) = "</vbox>"
		NodeType$(#Node_Gridbox) = "</gridbox>"
		NodeType$(#Node_Multibox) = "</multibox>"
		NodeType$(#Node_Singlebox) = "</singlebox>"
		
		; 		NodeType$(#Node_Button) = "</button>"
		; 		NodeType$(#Node_Buttonimage) = "</buttonimage>"
		; 		NodeType$(#Node_Calendar) = "</calendar>"
		; 		NodeType$(#Node_Canvas) = "</canvas>"
		; 		NodeType$(#Node_Checkbox) = "</checkbox>"
		; 		NodeType$(#Node_Combobox) = "</combobox>"
		NodeType$(#Node_Container) = "</container>"
		; 		NodeType$(#Node_Date) = "</date>"
		; 		NodeType$(#Node_Editor) = "</editor>"
		; 		NodeType$(#Node_Explorercombo) = "</explorercombo>"
		; 		NodeType$(#Node_Explorerlist) = "</explorerlist>"
		; 		NodeType$(#Node_Explorertree) = "</explorertree>"
		NodeType$(#Node_Frame) = "</frame>"
		; 		NodeType$(#Node_Hyperlink) = "</hyperlink>"
		; 		NodeType$(#Node_IPaddress) = "</ipaddress>"
		; 		NodeType$(#Node_Image) = "</image>"
		; 		NodeType$(#Node_Listicon) = "</listicon>"
		; 		NodeType$(#Node_Listview) = "</listview>"
		; 		NodeType$(#Node_Option) = "</option>"
		NodeType$(#Node_Panel) = "</panel>"
		; 		NodeType$(#Node_Progressbar) = "</progressbar>"
		NodeType$(#Node_Scrollarea) = "</scrollarea>"
		; 		NodeType$(#Node_Scrollbar) = "</scrollbar>"
		; 		NodeType$(#Node_Spin) = "</spin>"
		NodeType$(#Node_Splitter) = "</splitter>"
		; 		NodeType$(#Node_String) = "</string>"
		; 		NodeType$(#Node_Text) = "</text>"
		; 		NodeType$(#Node_Trackbar) = "</trackbar>"
		; 		NodeType$(#Node_Tree) = "</tree>"
		; 		NodeType$(#Node_Web) = "</web>"
		; 		NodeType$(#Node_Scintilla) = "</scintilla>"
		
		; 		NodeType$(#Node_Tab) = "</tab>"
		; 		NodeType$(#Node_Empty) = "</empty>"
		;<
	
	;> -----  MACROS      -  Diverse Macros für immer wieder verwendete Routinen um Parameter zu verarbeiten
		
		Macro Auswertung_Parameter_Date															; Macro für Parameter: Date (Datum)
			If Date <> #PB_Default And Date <>	0	:	XML$	+	" Date='"+Str(Date)+"'" : EndIf
		EndMacro
		Macro Auswertung_Parameter_ID																; Macro für Parameter: ID
			If ID <> #PB_Ignore			
				If ID = #PB_Any
					XML$	+	" id='#PB_Any'"
				Else
					XML$	+	" id='"+Str(ID)+"'"
				EndIf
			EndIf
		EndMacro
		Macro Auswertung_Parameter_ImageID														; Macro für Parameter: ImageID
			If ImageID <> #PB_Ignore
				If IsImage(ImageID) : ImageID = ImageID(ImageID) : EndIf
				XML$	+	" image='"+Str(ImageID)+"'"
			EndIf
		EndMacro
		Macro Auswertung_Parameter_Name															; Macro für Parameter: Name$
			If Len(Name$)	:	XML$	+	" name='"+T2H(Name$)+"'" : EndIf
		EndMacro
		Macro Auswertung_Parameter_Text															; Macro für Parameter: Text$
			If Len(Text$)	:	XML$	+	" text='"+T2H(Text$)+"'" : EndIf
		EndMacro
		Macro Auswertung_Parameter_Titel															; Macro für Parameter: Titel$
			If Len(Titel$)	:	XML$	+	" text='"+T2H(Titel$)+"'" : EndIf
		EndMacro
		Macro Auswertung_Parameter_Width															; Macro für Parameter: Width
			If Width <>	#PB_Default	And Width <> #PB_Ignore	:	XML$	+	" width='"+Str(Width)+"'" : EndIf
		EndMacro
		Macro Auswertung_Parameter_Height														; Macro für Parameter: Height
			If Height <> #PB_Default And Height <>	#PB_Ignore	:	XML$	+	" height='"+Str(Height)+"'" : EndIf
		EndMacro
		Macro Auswertung_Parameter_XmlParam														; Macro für Parameter: XmlParam$
			If Len(XmlParam$)	:	XML$	+	" "+XmlParam$ : EndIf
		EndMacro
		Macro Auswertung_Parameter_Spacing														; Macro für Parameter: Spacing
			If Spacing <> #PB_Default	: XML$ +	" spacing='"+Str(Spacing)+"'" : EndIf
		EndMacro
		Macro Auswertung_Parameter_Margin														; Macro für Parameter: Margin$
			If Margin$ = "" : Margin$ = "0" : EndIf
			If Margin$ <> #XmlDefaultMargin$ And Not FindString(UCase(Margin$),"DEFAULT")
				XML$ + " margin='"+Margin$+"'"
			EndIf
		EndMacro
		Macro Auswertung_Parameter_Expand														; Macro für Parameter: Expand
			If Expand = 0 : Expand = #PB_Default : EndIf
			If Expand <> #PB_Default																; Parameter: ColExpand
				If 		Expand = #Expand_Yes				: XML$	+	" expand="+#Expand_Yes$					; Expand = Yes
				ElseIf	Expand = #Expand_No				: XML$	+	" expand="+#Expand_No$					; Expand = No
				ElseIf	Expand = #Expand_Equal			: XML$	+	" expand="+#Expand_Equal$				; Expand = Equal
				ElseIf	Expand = #Expand_Vertical		: XML$	+	" expand="+#Expand_Vertical$			; Expand = Vertical
				ElseIf	Expand = #Expand_Horizontal	: XML$	+	" expand="+#Expand_Horizontal$		; Expand = Horizontal
				Else										: XML$	+	" expand="+#Expand_Item$+Str(Expand)+"'"	; Expand = Item:x
				EndIf
			EndIf
		EndMacro
		Macro Auswertung_Parameter_Align															; Macro für Parameter: Align
			Protected	Align$
			If Align = #PB_Default Or Align = #PB_Ignore : Align = 0 : EndIf
			
			If	Align & #alignTop		:	Align$ + "top,"		: EndIf
			If	Align & #alignBottom	:	Align$ + "bottom,"	: EndIf
			If Align & #alignLeft	:	Align$ + "left,"		: EndIf
			If Align & #alignRight	:	Align$ + "right,"		: EndIf
			If Align & #alignCenter	:	Align$ + "center,"	: EndIf
			If Right(Align$,1)="," : Align$=Left(Align$, Len(Align$)-1) : EndIf
			If Len(Align$) : XML$ + " align='"+Align$+"'" : EndIf
		EndMacro
		Macro Auswertung_Parameter_ScrollAreaWidth											; Macro für Parameter: ScrollAreaWidth
			If ScrollAreaWidth <>	#PB_Default	And ScrollAreaWidth <> #PB_Ignore	:	XML$	+	" ScrollAreaWidth='"+Str(ScrollAreaWidth)+"'" : EndIf
		EndMacro
		Macro Auswertung_Parameter_ScrollAreaHeight											; Macro für Parameter: ScrollAreaHeight
			If ScrollAreaHeight <>	#PB_Default	And ScrollAreaHeight <> #PB_Ignore	:	XML$	+	" ScrollAreaHeight='"+Str(ScrollAreaHeight)+"'" : EndIf
		EndMacro
		Macro Auswertung_Parameter_ScrollStep													; Macro für Parameter: ScrollStep
			If ScrollStep <>	#PB_Default	And ScrollStep <> #PB_Ignore	:	XML$	+	" ScrollStep='"+Str(ScrollStep)+"'" : EndIf
		EndMacro
		Macro Auswertung_Parameter_Gadget1													; Macro für Parameter: Gadget1
			If Gadget1 <>	#PB_Default	And Gadget1 <> #PB_Ignore	:	XML$	+	" Gadget1='"+Str(Gadget1)+"'" : EndIf
		EndMacro
		Macro Auswertung_Parameter_Gadget2													; Macro für Parameter: Gadget2
			If Gadget2 <>	#PB_Default	And Gadget2 <> #PB_Ignore	:	XML$	+	" Gadget2='"+Str(Gadget2)+"'" : EndIf
		EndMacro
		;<
	
	;> -----  PROZEDUREN  -  Diverse Prozeduren, u.A. zum Bearbeiten des XML-Textes 
		
		Procedure.s		Indent(AdditionalIndent=0)					; erzeugt die Formatierungs-Leerzeichen bzw. -Tabs zum Einrücken des XML-Textes
			Protected Ind$, n
			If XMLout_Indent > 0 And ListSize(NodeTree()) > 0
				If XMLout_UseRealTab
					For n = 1 To XMLout_Indent * (ListSize(NodeTree()) + AdditionalIndent)
						Ind$ + Chr(9)
					Next
				Else
					Ind$ = Space(XMLout_Indent * (ListSize(NodeTree()) + AdditionalIndent))
				EndIf
			EndIf
			ProcedureReturn Ind$
		EndProcedure
		Procedure		SetXMLOutputFormat(Attribut, Value)		; zum Setzen versch. Parameter für den erzeucgten XML-Code
			Select Attribut
				Case #XMLout_Indent
					XMLout_Indent = Value
				Case #XMLout_LineBreak
					XMLout_LineBreak = Value
				Case #XMLout_UseRealTab
					XMLout_UseRealTab = Value
				Case #XMLout_AlignLineBreak
					XMLout_AlignLineBreak = Value
			EndSelect
		EndProcedure
		
		Procedure.s		Align(XML_Code$, Align=0, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, ForceZeroMargin=#False)
			
			Protected Col$, Row$, vBoxText$, vAlign$, hBoxText$, hAlign$, EndText$, n, ActPos, CR$, AddIndent, FinalXML$
			
			If ColSpan <> #PB_Ignore : Col$ = " colspan='"+Str(ColSpan)+"'" : EndIf
			If RowSpan <> #PB_Ignore : Row$ = " rowspan='"+Str(RowSpan)+"'" : EndIf
			;If Margin$ = "" : Margin$ = "0" : EndIf
			If Margin$ <> "" And Margin$ <> #XmlDefaultMargin$ And Not FindString(UCase(Margin$),"Default")
				Margin$ = " margin='"+Margin$+"'"
			Else
				Margin$ = ""
			EndIf
			
			If XMLout_AlignLineBreak : CR$ = Chr(13) : EndIf
			
			If Align = #PB_Default Or Align = #PB_Ignore : Align = 0 : EndIf
			
			If Align & #alignTop		: vAlign$ + "top"		: EndIf
			If Align & #alignBottom	: vAlign$ + "bottom"	: EndIf
			If Align & #alignCenter And Not Align & (#alignTop | #alignBottom) : vAlign$ = "center" : EndIf
			
			If Align & #alignLeft	: hAlign$ + "left"	: EndIf
			If Align & #alignRight	: hAlign$ + "right"	: EndIf
			If Align & #alignCenter And Not Align & (#alignLeft | #alignRight) : hAlign$ = "center" : EndIf
			
			If Len(vAlign$) Or ForceZeroMargin										; SingleBox mit vertikaler Expansion für top/bottom Ausrichtung
				If XMLout_AlignLineBreak : EndText$ + Indent() : EndIf
				vBoxText$ + "<singlebox expand='horizontal' align='"+vAlign$+"'"+Margin$+Col$+Row$+">"+CR$
				EndText$	+ "</singlebox>"+CR$
				Margin$	= " margin='0'"
				Col$		= ""
				Row$		= ""
				AddIndent + 1
			EndIf
			If Len(hAlign$) Or ForceZeroMargin			; SingleBox mit horizontaler Expansion für left/right Ausrichtung
				If AddIndent : hBoxText$ + Indent(AddIndent) : EndIf
				hBoxText$ + "<singlebox expand='vertical' align='"+hAlign$+"'"+Margin$+Col$+Row$+">"+CR$
				EndText$	= "</singlebox>"+CR$+EndText$
				If XMLout_AlignLineBreak : EndText$ = Indent(AddIndent) + EndText$ : EndIf
				Margin$	= ""
				Col$		= ""
				Row$		= ""
				AddIndent + 1
			EndIf
			If Len(Col$ + Row$ + Margin$)					; Wenn keine Singlebox erstellt wurde, dann ggfl. Colspan oder Rowspan in übergebenen XML-Text injizieren.
				For n = Len(XML_Code$) To 1 Step -1
					If Mid(XML_Code$, n, 1)=">"
						If Mid(XML_Code$, n-1, 2)="/>" : n-1 : EndIf
						XML_Code$ = Left(XML_Code$,n-1) + Col$ + Row$ + Margin$ + Mid(XML_Code$, n)
						Break
					EndIf
				Next
			EndIf
			
			If AddIndent
				XML_Code$ = Indent(AddIndent) + XML_Code$
			EndIf
			
			FinalXML$ = vBoxText$+hBoxText$+XML_Code$+CR$+EndText$
			If Right(FinalXML$,1) = Chr(13)
				FinalXML$ = Left(FinalXML$, Len(FinalXML$)-1)
			EndIf
			
			ProcedureReturn FinalXML$
			
		EndProcedure
		Procedure.s		T2H(Text$)										; konvertiert alle nicht unterstützten ASCii-Zeichen in gültigen HTML-Code
			Protected Pos=1
			Text$ = ReplaceString(Text$, "&", "&amp;")
			Text$ = ReplaceString(Text$, Chr(9), "&#9;")
			Text$ = ReplaceString(Text$, Chr(10), "&#10;")
			Text$ = ReplaceString(Text$, Chr(39), "&#39;")
			Text$ = ReplaceString(Text$, Chr(60), "&lt;")
			Text$ = ReplaceString(Text$, Chr(62), "&gt;")
			While pos < Len(Text$)
				If Asc(Mid(Text$,Pos,1)) < 32
					Text$ = Left(Text$,Pos-1)+Mid(Text$,pos+1)
				Else
					Pos + 1
				EndIf
			Wend
			ProcedureReturn Text$
		EndProcedure
		
		Procedure		ShowNodeErrors(State)						; zum de-/aktiviert von Nachrichten, die bei Fehlern von Close-Nodes angezeigt werden
			ShowNodeErrors = State
		EndProcedure
		Procedure		ShowNodeErrorMessage(NodeType)			; zeigt eine Close-Node Fehlermeldung an
			
			Protected	ChkNode$
			
			If NodeType < #Node_FirstNode Or NodeType > #Node_LastNode 			; Wenn übergebener NodeType ok ist, dann lese Definition, ansonsten zeige '<??? unknown ???>'
				ChkNode$ = "<??? unknown ???>"
			Else
				ChkNode$ = NodeType$(NodeType)+" ("+Str(NodeType)+")"
			EndIf
			
			Protected	Msg$ = "Trying To close NodeType: '"+ChkNode$+"'"+CR$+CR$
			
			If LastElement(NodeTree())
				Msg$ + "but NodeType: '"+NodeType$(NodeTree())+"' should be closed instead of '"+ChkNode$+"'."
			Else
				Msg$ + "but NodeList is empty."
			EndIf
			
			MessageRequester("Close-Node Error !!!", Msg$, #PB_MessageRequester_Ok)
			
		EndProcedure
		Procedure		CheckLastNode(NodeType, ShowError = #PB_Default)		; returns #False, if last Node is NOT = NodeType, and #True, if last Node = NodeType
			
			If ShowError = #PB_Default : ShowError = ShowNodeErrors : EndIf
			
			If LastElement(NodeTree())
				If NodeType = NodeTree()
					ProcedureReturn #True
				EndIf
			EndIf
			
			If ShowError <> #NodeErrorMessage_None
				ShowNodeErrorMessage(NodeType)
			EndIf
			
		EndProcedure
		
		Procedure		ClearXML(ClearNodeList=#True)				; Löscht den XML-Buffer - setzt also die XML-Definition und ggfl. die Node-Liste zurück
			XMLBuffer$ = ""
			If ClearNodeList
				ClearList(NodeTree())
			EndIf
		EndProcedure
		Procedure.s		GetXML()											; 
			ProcedureReturn	XMLBuffer$
		EndProcedure
		Procedure		SetXML(XML_Text$, XmlConvert=#False)	; 
			If XmlConvert
				XMLBuffer$ = T2H(XML_Text$)
			Else
				XMLBuffer$ = XML_Text$
			EndIf
		EndProcedure
		Procedure.s		AddXML(XML_Text$, XmlConvert=#False)	; fügt den übergebenen XML-Text dem XML-Puffer hinzu.
			If XmlConvert
				XML_Text$ = T2H(XML_Text$)
			EndIf
			
			If XMLout_Indent : XML_Text$ = Trim(XML_Text$) : EndIf
			
			XMLBuffer$ + Indent() + XML_Text$ + CR$
			LastXMLLine$ + XML_Text$ + CR$
			ProcedureReturn Indent() + XML_Text$
		EndProcedure
		Procedure.s		CloseNode(NodeType = #PB_Any)				; schließt die in NodeType vorgegebene Node, oder die Node des letzten Containers (letzter Listen-Eintrags in NodeTree())
			
			Protected	Node$, XML$
			
			If LastElement(NodeTree())
				If NodeType = #PB_Any
					NodeType = NodeTree()
				EndIf
			EndIf
			
			If NodeType < #Node_FirstNode Or NodeType > #Node_LastNode 			; Wenn übergebener NodeType ok ist, dann lese Definition, ansonsten zeige '<??? unknown ???>'
				XML$ = "</??? unknown node-type: '"+Str(NodeType)+"' ???>"
			Else
				XML$ = NodeType$(NodeType)
			EndIf
			
			If ListSize(NodeTree()) > 0
				DeleteElement(NodeTree(),1)
			EndIf
			
			XMLBuffer$ + Indent() + XML$ + CR$
			LastXMLLine$ + XML$ + CR$
			
			ProcedureReturn Indent() + XML$
			
		EndProcedure
		;<
		;> -----  PROZEDUREN  -  XML-Layout Elemente
		
		Procedure.s		Dialogs()
			
			Protected XML$ = "<dialogs>"
			
			XMLBuffer$		+ Indent() + XML$ + CR$
			LastXMLLine$	= XML$ + CR$			; Store last XML-Line, to show in Error-Message
			
			LastElement(NodeTree()) : AddElement(NodeTree()) : NodeTree() = #Node_Dialog		; Element "Window" der Node-Liste hinzufügen
			
			ProcedureReturn Indent() + XML$
			
		EndProcedure
		Procedure.s		Window(ID, Name$, Titel$, Flags=0, Width=#PB_Ignore, Height=#PB_Ignore, XmlParam$="")
			
			Protected XML$ = "<window "
			
			Auswertung_Parameter_ID																				; Parameter: ID
			Auswertung_Parameter_Name																			; Parameter: Name$
			Auswertung_Parameter_Titel																			; Parameter: Titel$
			Auswertung_Parameter_Width																			; Parameter: Width
			Auswertung_Parameter_Height																		; Parameter: Height
			Auswertung_Parameter_XmlParam																		; Parameter: XmlParam$
			
			If Flags																									; Parameter: Flags
				
				XML$ + " flags='"
				
				If flags & #PB_Window_SystemMenu			= #PB_Window_SystemMenu			: XML$ + "#PB_Window_SystemMenu | "			: EndIf
				If flags & #PB_Window_MinimizeGadget	= #PB_Window_MinimizeGadget	: XML$ + "#PB_Window_MinimizeGadget | "	: EndIf
				If flags & #PB_Window_MaximizeGadget	= #PB_Window_MaximizeGadget	: XML$ + "#PB_Window_MaximizeGadget | "	: EndIf
				If flags & #PB_Window_SizeGadget			= #PB_Window_SizeGadget			: XML$ + "#PB_Window_SizeGadget | "			: EndIf
				If flags & #PB_Window_Invisible			= #PB_Window_Invisible			: XML$ + "#PB_Window_Invisible | "			: EndIf
				If flags & #PB_Window_TitleBar			= #PB_Window_TitleBar			: XML$ + "#PB_Window_TitleBar | "			: EndIf
				If flags & #PB_Window_Tool					= #PB_Window_Tool					: XML$ + "#PB_Window_Tool | "					: EndIf
				If flags & #PB_Window_BorderLess			= #PB_Window_BorderLess			: XML$ + "#PB_Window_BorderLess | "			: EndIf
				If flags & #PB_Window_ScreenCentered	= #PB_Window_ScreenCentered	: XML$ + "#PB_Window_ScreenCentered | "	: EndIf
				If flags & #PB_Window_WindowCentered	= #PB_Window_WindowCentered	: XML$ + "#PB_Window_WindowCentered | "	: EndIf
				If flags & #PB_Window_Maximize			= #PB_Window_Maximize			: XML$ + "#PB_Window_Maximize | "			: EndIf
				If flags & #PB_Window_Minimize			= #PB_Window_Minimize			: XML$ + "#PB_Window_Minimize | "			: EndIf
				If flags & #PB_Window_NoGadgets			= #PB_Window_NoGadgets			: XML$ + "#PB_Window_NoGadgets | "			: EndIf
				If flags & #PB_Window_NoActivate			= #PB_Window_NoActivate			: XML$ + "#PB_Window_NoActivate | "			: EndIf
				If Right(XML$,3)=" | " : XML$ = Left(XML$, Len(XML$)-3) : EndIf
				XML$ + "'"
				
			EndIf
			
			XML$ + ">"
			
			XMLBuffer$		+ Indent() + XML$ + CR$
			LastXMLLine$	= XML$ + CR$			; Store last XML-Line, to show in Error-Message
			
			LastElement(NodeTree()) : AddElement(NodeTree()) : NodeTree() = #Node_Window		; Element "Window" der Node-Liste hinzufügen
			
			ProcedureReturn Indent() + XML$
			
		EndProcedure
		Procedure.s		vBox(Expand=#PB_Default, Align=#PB_Default, Spacing=#PB_Default, XmlParam$="")
			
			Protected XML$ = "<vbox"
			
			Auswertung_Parameter_Expand																						; Parameter: Expand
			Auswertung_Parameter_Align																							; Parameter: Align
			Auswertung_Parameter_Spacing																						; Parameter: Spacing
			Auswertung_Parameter_XmlParam																						; Parameter: XmlParam$
			
			XML$ + ">" 
			
			XMLBuffer$		+ Indent() + XML$ + CR$
			LastXMLLine$	= XML$ + CR$			; Store last XML-Line, to show in Error-Message
			
			LastElement(NodeTree()) : AddElement(NodeTree()) : NodeTree() = #Node_VBox			; Element "vBox" der Node-Liste hinzufügen
			
			ProcedureReturn Indent() + XML$
			
		EndProcedure
		Procedure.s		hBox(Expand=#PB_Default, Align=#PB_Default, Spacing=#PB_Default, XmlParam$="")
			
			Protected XML$ = "<hbox"
			
			Auswertung_Parameter_Expand																						; Parameter: Expand
			Auswertung_Parameter_Align																							; Parameter: Align
			Auswertung_Parameter_Spacing																						; Parameter: Spacing
			Auswertung_Parameter_XmlParam																						; Parameter: XmlParam$
			
			XML$ + ">" 
			
			XMLBuffer$		+ Indent() + XML$ + CR$
			LastXMLLine$	= XML$ + CR$			; Store last XML-Line, to show in Error-Message
			
			LastElement(NodeTree()) : AddElement(NodeTree()) : NodeTree() = #Node_HBox			; Element "hBox" der Node-Liste hinzufügen
			
			ProcedureReturn Indent() + XML$
			
		EndProcedure
		Procedure.s		GridBox(Columns=#PB_Default, ColSpacing=#PB_Default, RowSpacing=#PB_Default, ColExpand=#PB_Default, RowExpand=#PB_Default, XmlParam$="")
			
			Protected XML$ = "<gridbox"
			
			If Columns <> #PB_Default		: XML$	+	" columns='"+Str(Columns)+"'"			: EndIf				; Parameter: Columns
			If ColSpacing <> #PB_Default	: XML$	+	" colspacing='"+Str(ColSpacing)+"'" : EndIf				; Parameter: ColSpacing
			If RowSpacing <> #PB_Default	: XML$	+	" rowspacing='"+Str(RowSpacing)+"'" : EndIf				; Parameter: RowSpacing
			If ColExpand <> #PB_Default																							; Parameter: ColExpand
				If 		ColExpand = #Expand_Yes		: XML$	+	" colexpand="+#Expand_Yes$							; Expand = Yes
				ElseIf	ColExpand = #Expand_No		: XML$	+	" colexpand="+#Expand_No$							; Expand = No
				ElseIf	ColExpand = #Expand_Equal	: XML$	+	" colexpand="+#Expand_Equal$						; Expand = Equal
				Else											: XML$	+	" colexpand="+#Expand_Item$+Str(ColExpand)+"'"				; Expand = Item:x
				EndIf
			EndIf
			If RowExpand <> #PB_Default																							; Parameter: RowExpand
				If 		RowExpand = #Expand_Yes		: XML$	+	" rowexpand="+#Expand_Yes$							; Expand = Yes
				ElseIf	RowExpand = #Expand_No		: XML$	+	" rowexpand="+#Expand_No$							; Expand = No
				ElseIf	RowExpand = #Expand_Equal	: XML$	+	" rowexpand="+#Expand_Equal$						; Expand = Equal
				Else											: XML$	+	" rowexpand="+#Expand_Item$+Str(RowExpand)+"'"				; Expand = Item:x
				EndIf
			EndIf
			
			Auswertung_Parameter_XmlParam																							; Parameter: XmlParam$
			
			XML$ + ">" 
			
			XMLBuffer$		+ Indent() + XML$ + CR$
			LastXMLLine$	= XML$ + CR$			; Store last XML-Line, to show in Error-Message
			
			LastElement(NodeTree()) : AddElement(NodeTree()) : NodeTree() = #Node_Gridbox		; Element "GridBox" der Node-Liste hinzufügen
			
			ProcedureReturn Indent() + XML$
			
		EndProcedure
		Procedure.s		MultiBox(Expand=#PB_Default, Align=#PB_Default, Margin$="0", XmlParam$="")
			
			Protected XML$ = "<multibox"
			
			Auswertung_Parameter_Expand																						; Parameter: Expand
			Auswertung_Parameter_Align																							; Parameter: Align
			Auswertung_Parameter_Margin																						; Parameter: Margin$
			Auswertung_Parameter_XmlParam																						; Parameter: XmlParam$
			
			XML$ + ">" 
			
			XMLBuffer$		+ Indent() + XML$ + CR$
			LastXMLLine$	= XML$ + CR$																			; Store last XML-Line, to show in Error-Message
			
			LastElement(NodeTree()) : AddElement(NodeTree()) : NodeTree() = #Node_Multibox		; Element "Multibox" der Node-Liste hinzufügen
			
			ProcedureReturn Indent() + XML$
			
		EndProcedure
		Procedure.s		SingleBox(Expand=#PB_Default, Align=#PB_Default, Margin$="0", Expandwidth=0, Expandheight=0, XmlParam$="")
			
			Protected XML$ = "<singlebox"
			
			Auswertung_Parameter_Expand																						; Parameter: Expand
			Auswertung_Parameter_Align																							; Parameter: Align
			Auswertung_Parameter_Margin																						; Parameter: Margin$
			
			If Expandwidth <> 0	: XML$ + " Expandwidth='"+Str(Expandwidth)+"'"		: EndIf					; Parameter: Expandwidth
			
			If Expandheight <> 0	: XML$ + " Expandheight='"+Str(Expandheight)+"'"	: EndIf					; Parameter: Expandheight
			
			Auswertung_Parameter_XmlParam																						; Parameter: XmlParam$
			
			XML$ + ">" 
			
			XMLBuffer$		+ Indent() + XML$ + CR$
			LastXMLLine$	= XML$ + CR$																			; Store last XML-Line, to show in Error-Message
			
			LastElement(NodeTree()) : AddElement(NodeTree()) : NodeTree() = #Node_Singlebox		; Element "Singlebox" der Node-Liste hinzufügen
			
			ProcedureReturn Indent() + XML$
			
		EndProcedure
		Procedure.s		Empty(Width=#PB_Default, Height=#PB_Default, XmlParam$="")
			
			Protected XML$ = "<empty"
			
			Auswertung_Parameter_Width																			; Parameter: Width
			Auswertung_Parameter_Height																		; Parameter: Height
			Auswertung_Parameter_XmlParam																		; Parameter: XmlParam$
			
			XML$ + "/>"
			
			;XML$ = Align(XML$, Align, Margin$, ColSpan, RowSpan)
			
			XMLBuffer$		+ Indent() + XML$ + CR$
			LastXMLLine$	= XML$ + CR$			; Store last XML-Line, to show in Error-Message
			
			ProcedureReturn Indent() + XML$
			
		EndProcedure
		
		;<
		;> -----  PROZEDUREN  -  XML-Layout Elemente  (Close-Nodes)
		
		Procedure.s		EndDialogs()
			CheckLastNode(#Node_Dialog)
			ProcedureReturn	CloseNode(#Node_Dialog)
		EndProcedure
		Procedure.s		EndWindow()
			CheckLastNode(#Node_Window)
			ProcedureReturn	CloseNode(#Node_Window)
		EndProcedure
		Procedure.s		EndVBox()
			CheckLastNode(#Node_VBox)
			ProcedureReturn	CloseNode(#Node_VBox)
		EndProcedure
		Procedure.s		EndHBox()
			CheckLastNode(#Node_HBox)
			ProcedureReturn	CloseNode(#Node_HBox)
		EndProcedure
		Procedure.s		EndGridBox()
			CheckLastNode(#Node_Gridbox)
			ProcedureReturn	CloseNode(#Node_Gridbox)
		EndProcedure
		Procedure.s		EndMultiBox()
			CheckLastNode(#Node_Multibox)
			ProcedureReturn	CloseNode(#Node_Multibox)
		EndProcedure
		Procedure.s		EndSingleBox()
			CheckLastNode(#Node_Singlebox)
			ProcedureReturn	CloseNode(#Node_Singlebox)
		EndProcedure
		
		;<
	
	;> -----  PROZEDUREN  -  PB Container-Elemente
		
		Procedure.s		Container(ID=#PB_Ignore, Name$="", Flags=0, Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
			
			Protected XML$ = "<container"
			
			Auswertung_Parameter_ID																				; Parameter: ID
			Auswertung_Parameter_Name																			; Parameter: Name$
			Auswertung_Parameter_Width																			; Parameter: Width
			Auswertung_Parameter_Height																		; Parameter: Height
			Auswertung_Parameter_XmlParam																		; Parameter: XmlParam$
			
			If Flags																									; Parameter: Flags
				
				XML$ + " flags='"
				
				If flags & #PB_Container_Flat			= #PB_Container_Flat			: XML$ + "#PB_Container_Flat | "			: EndIf
				If flags & #PB_Container_Raised		= #PB_Container_Raised		: XML$ + "#PB_Container_Raised | "		: EndIf
				If flags & #PB_Container_Single		= #PB_Container_Single		: XML$ + "#PB_Container_Single | "		: EndIf
				If flags & #PB_Container_Double		= #PB_Container_Double		: XML$ + "#PB_Container_Double | "		: EndIf
				If Right(XML$,3)=" | " : XML$ = Left(XML$, Len(XML$)-3) : EndIf
				XML$ + "'"
				
			EndIf
			
			XML$ + ">"
			
			XML$ = Align(XML$, Align, Margin$, ColSpan, RowSpan)
			
			XMLBuffer$		+ Indent() + XML$ + CR$
			LastXMLLine$	= XML$ + CR$			; Store last XML-Line, to show in Error-Message
			
			LastElement(NodeTree()) : AddElement(NodeTree()) : NodeTree() = #Node_Container		; Element "Container" der Node-Liste hinzufügen
			
			ProcedureReturn Indent() + XML$
			
		EndProcedure
		Procedure.s		Frame(ID=#PB_Ignore, Name$="", Text$="", Flags=0, Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
			
			Protected XML$ = "<frame"
			
			Auswertung_Parameter_ID																				; Parameter: ID
			Auswertung_Parameter_Name																			; Parameter: Name$
			Auswertung_Parameter_Text																			; Parameter: Text$
			Auswertung_Parameter_Width																			; Parameter: Width
			Auswertung_Parameter_Height																		; Parameter: Height
			Auswertung_Parameter_XmlParam																		; Parameter: XmlParam$
			
			If Flags																									; Parameter: Flags
				
				XML$ + " flags='"
				
				If flags & #PB_Frame_Single			= #PB_Frame_Single			: XML$ + "#PB_Frame_Single | "			: EndIf
				If flags & #PB_Frame_Double			= #PB_Frame_Double			: XML$ + "#PB_Frame_Double | "			: EndIf
				If flags & #PB_Frame_Flat				= #PB_Frame_Flat				: XML$ + "#PB_Frame_Flat | "				: EndIf
				If Right(XML$,3)=" | " : XML$ = Left(XML$, Len(XML$)-3) : EndIf
				XML$ + "'"
				
			EndIf
			
			XML$ + ">"
			
			XML$ = Align(XML$, Align, Margin$, ColSpan, RowSpan)
			
			XMLBuffer$		+ Indent() + XML$ + CR$
			LastXMLLine$	= XML$ + CR$			; Store last XML-Line, to show in Error-Message
			
			LastElement(NodeTree()) : AddElement(NodeTree()) : NodeTree() = #Node_Frame		; Element "Frame" der Node-Liste hinzufügen
			
			ProcedureReturn Indent() + XML$
			
		EndProcedure
		Procedure.s		Panel(ID=#PB_Ignore, Name$="", Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
			
			Protected XML$ = "<panel"
			
			Auswertung_Parameter_ID																				; Parameter: ID
			Auswertung_Parameter_Name																			; Parameter: Name$
			Auswertung_Parameter_Width																			; Parameter: Width
			Auswertung_Parameter_Height																		; Parameter: Height
			Auswertung_Parameter_XmlParam																		; Parameter: XmlParam$
			
			XML$ + ">"
			
			XML$ = Align(XML$, Align, Margin$, ColSpan, RowSpan)
			
			XMLBuffer$		+ Indent() + XML$ + CR$
			LastXMLLine$	= XML$ + CR$			; Store last XML-Line, to show in Error-Message
			
			LastElement(NodeTree()) : AddElement(NodeTree()) : NodeTree() = #Node_Panel		; Element "Panel" der Node-Liste hinzufügen
			
			ProcedureReturn Indent() + XML$
			
		EndProcedure
		Procedure.s		ScrollArea(ID=#PB_Ignore, Name$="", ScrollAreaWidth=#PB_Default, ScrollAreaHeight=#PB_Default, ScrollStep=#PB_Default, Flags=0, Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
			
			Protected XML$ = "<scrollarea"
			
			Auswertung_Parameter_ID																				; Parameter: ID
			Auswertung_Parameter_Name																			; Parameter: Name$
			Auswertung_Parameter_ScrollAreaWidth
			Auswertung_Parameter_ScrollAreaHeight
			Auswertung_Parameter_ScrollStep
			Auswertung_Parameter_Width																			; Parameter: Width
			Auswertung_Parameter_Height																		; Parameter: Height
			Auswertung_Parameter_XmlParam																		; Parameter: XmlParam$
			
			XML$ + ">"
			
			XML$ = Align(XML$, Align, Margin$, ColSpan, RowSpan)
			
			XMLBuffer$		+ Indent() + XML$ + CR$
			LastXMLLine$	= XML$ + CR$			; Store last XML-Line, to show in Error-Message
			
			LastElement(NodeTree()) : AddElement(NodeTree()) : NodeTree() = #Node_Scrollarea		; Element "ScrollArea" der Node-Liste hinzufügen
			
			ProcedureReturn Indent() + XML$
			
		EndProcedure
		Procedure.s		Splitter(ID, Name$, Gadget1, Gadget2, Flags=0, Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
			
			Protected XML$ = "<splitter"
			
			Auswertung_Parameter_ID																				; Parameter: ID
			Auswertung_Parameter_Name																			; Parameter: Name$
			Auswertung_Parameter_Gadget1
			Auswertung_Parameter_Gadget2
			Auswertung_Parameter_Width																			; Parameter: Width
			Auswertung_Parameter_Height																		; Parameter: Height
			Auswertung_Parameter_XmlParam																		; Parameter: XmlParam$
			
			If Flags																									; Parameter: Flags
				
				XML$ + " flags='"
				
				If flags & #PB_Splitter_Vertical			= #PB_Splitter_Vertical			: XML$ + "#PB_Splitter_Vertical | "			: EndIf
				If flags & #PB_Splitter_Separator		= #PB_Splitter_Separator		: XML$ + "#PB_Splitter_Separator | "		: EndIf
				If flags & #PB_Splitter_FirstFixed		= #PB_Splitter_FirstFixed		: XML$ + "#PB_Splitter_FirstFixed | "		: EndIf
				If flags & #PB_Splitter_SecondFixed		= #PB_Splitter_SecondFixed		: XML$ + "#PB_Splitter_SecondFixed | "		: EndIf
				If Right(XML$,3)=" | " : XML$ = Left(XML$, Len(XML$)-3) : EndIf
				XML$ + "'"
				
			EndIf
			
			XML$ + ">"
			
			XML$ = Align(XML$, Align, Margin$, ColSpan, RowSpan)
			
			XMLBuffer$		+ Indent() + XML$ + CR$
			LastXMLLine$	= XML$ + CR$			; Store last XML-Line, to show in Error-Message
			
			LastElement(NodeTree()) : AddElement(NodeTree()) : NodeTree() = #Node_Splitter		; Element "Splitter" der Node-Liste hinzufügen
			
			ProcedureReturn Indent() + XML$
			
		EndProcedure
		
		Procedure.s		EndContainer()
			CheckLastNode(#Node_Container)
			ProcedureReturn	CloseNode(#Node_Container)
		EndProcedure
		Procedure.s		EndFrame()
			CheckLastNode(#Node_Frame)
			ProcedureReturn	CloseNode(#Node_Frame)
		EndProcedure
		Procedure.s		EndPanel()
			CheckLastNode(#Node_Panel)
			ProcedureReturn	CloseNode(#Node_Panel)
		EndProcedure
		Procedure.s		EndScrollArea()
			CheckLastNode(#Node_Scrollarea)
			ProcedureReturn	CloseNode(#Node_Scrollarea)
		EndProcedure
		Procedure.s		EndSplitter()
			CheckLastNode(#Node_Splitter)
			ProcedureReturn	CloseNode(#Node_Splitter)
		EndProcedure
		
		;<
	
	;> -----  PROZEDUREN  -  Procedures() zum Wrappen der PB Gadget-Elemente
		
		Procedure.s		Button(ID=#PB_Ignore, Name$="", Text$="", Flags=0, Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
			
			Protected XML$ = "<button"
			
			Auswertung_Parameter_ID																				; Parameter: ID
			Auswertung_Parameter_Name																			; Parameter: Name$
			Auswertung_Parameter_Text																			; Parameter: Text$
			Auswertung_Parameter_Width																			; Parameter: Width
			Auswertung_Parameter_Height																		; Parameter: Height
			Auswertung_Parameter_XmlParam																		; Parameter: XmlParam$
			
			If Flags																									; Parameter: Flags
				
				XML$ + " flags='"
				
				If flags & #PB_Button_Right		: XML$ + "#PB_Button_Right | "		: EndIf
				If flags & #PB_Button_Left			: XML$ + "#PB_Button_Left | "			: EndIf
				If flags & #PB_Button_Default		: XML$ + "#PB_Button_Default | "		: EndIf
				If flags & #PB_Button_MultiLine	: XML$ + "#PB_Button_MultiLine | "	: EndIf
				If flags & #PB_Button_Toggle		: XML$ + "#PB_Button_Toggle | "		: EndIf
				If Right(XML$,3)=" | " : XML$ = Left(XML$, Len(XML$)-3) : EndIf
				XML$ + "'"
				
			EndIf
			
			XML$ + "/>"
			
			XML$ = Align(XML$, Align, Margin$, ColSpan, RowSpan)
			
			XMLBuffer$		+ Indent() + XML$ + CR$
			LastXMLLine$	= XML$ + CR$			; Store last XML-Line, to show in Error-Message
			
			ProcedureReturn Indent() + XML$
			
		EndProcedure
		Procedure.s		ButtonImage(ID=#PB_Ignore, Name$="", ImageID=0 , Flags=0, Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
			
			Protected XML$ = "<buttonimage"
			
			Auswertung_Parameter_ID																				; Parameter: ID
			Auswertung_Parameter_Name																			; Parameter: Name$
			Auswertung_Parameter_ImageID																		; Parameter: ImageID
			Auswertung_Parameter_Width																			; Parameter: Width
			Auswertung_Parameter_Height																		; Parameter: Height
			Auswertung_Parameter_XmlParam																		; Parameter: XmlParam$
			
			If Flags	& #PB_Button_Toggle = #PB_Button_Toggle											; Parameter: Flags
				XML$ + " flags='#PB_Button_Right'"
			EndIf
			
			XML$ + "/>"
			
			XML$ = Align(XML$, Align, Margin$, ColSpan, RowSpan)
			
			XMLBuffer$		+ Indent() + XML$ + CR$
			LastXMLLine$	= XML$ + CR$			; Store last XML-Line, to show in Error-Message
			
			ProcedureReturn Indent() + XML$
			
		EndProcedure
		Procedure.s		Calendar(ID=#PB_Ignore, Name$="", Date=#PB_Default , Flags=0, Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
			
			Protected XML$ = "<buttonimage"
			
			Auswertung_Parameter_ID																				; Parameter: ID
			Auswertung_Parameter_Name																			; Parameter: Name$
			Auswertung_Parameter_Date																			; Parameter: Date (Datum)
			Auswertung_Parameter_Width																			; Parameter: Width
			Auswertung_Parameter_Height																		; Parameter: Height
			Auswertung_Parameter_XmlParam																		; Parameter: XmlParam$
			
			If Flags	& #PB_Calendar_Borderless = #PB_Calendar_Borderless							; Parameter: Flags
				XML$ + " flags='#PB_Calendar_Borderless'"
			EndIf
			
			XML$ + "/>"
			
			XML$ = Align(XML$, Align, Margin$, ColSpan, RowSpan)
			
			XMLBuffer$		+ Indent() + XML$ + CR$
			LastXMLLine$	= XML$ + CR$			; Store last XML-Line, to show in Error-Message
			
			ProcedureReturn Indent() + XML$
			
		EndProcedure
		Procedure.s		Canvas(ID=#PB_Ignore, Name$="", Flags=0, Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
			
			Protected XML$ = "<canvas"
			
			Auswertung_Parameter_ID																				; Parameter: ID
			Auswertung_Parameter_Name																			; Parameter: Name$
			Auswertung_Parameter_Width																			; Parameter: Width
			Auswertung_Parameter_Height																		; Parameter: Height
			Auswertung_Parameter_XmlParam																		; Parameter: XmlParam$
			
			If Flags																									; Parameter: Flags
				
				XML$ + " flags='"
				
				If flags & #PB_Canvas_Border		= #PB_Canvas_Border		: XML$ + "#PB_Canvas_Border | "		: EndIf
				If flags & #PB_Canvas_ClipMouse	= #PB_Canvas_ClipMouse	: XML$ + "#PB_Canvas_ClipMouse | "	: EndIf
				If flags & #PB_Canvas_Keyboard	= #PB_Canvas_Keyboard	: XML$ + "#PB_Canvas_Keyboard | "	: EndIf
				If flags & #PB_Canvas_DrawFocus	= #PB_Canvas_DrawFocus	: XML$ + "#PB_Canvas_DrawFocus | "	: EndIf
				If Right(XML$,3)=" | " : XML$ = Left(XML$, Len(XML$)-3) : EndIf
				XML$ + "'"
				
			EndIf
			
			XML$ + "/>"
			
			XML$ = Align(XML$, Align, Margin$, ColSpan, RowSpan)
			
			XMLBuffer$		+ Indent() + XML$ + CR$
			LastXMLLine$	= XML$ + CR$			; Store last XML-Line, to show in Error-Message
			
			ProcedureReturn Indent() + XML$
			
		EndProcedure
		Procedure.s		CheckBox(ID=#PB_Ignore, Name$="", Text$="", Flags=0, Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
			
			Protected XML$ = "<checkbox"
			
			Auswertung_Parameter_ID																				; Parameter: ID
			Auswertung_Parameter_Name																			; Parameter: Name$
			Auswertung_Parameter_Text																			; Parameter: Text$
			Auswertung_Parameter_Width																			; Parameter: Width
			Auswertung_Parameter_Height																		; Parameter: Height
			Auswertung_Parameter_XmlParam																		; Parameter: XmlParam$
			
			If Flags																									; Parameter: Flags
				
				XML$ + " flags='"
				
				If flags & #PB_CheckBox_Right				: XML$ + "#PB_CheckBox_Right | "				: EndIf
				If flags & #PB_CheckBox_Center			: XML$ + "#PB_CheckBox_Center | "			: EndIf
				If flags & #PB_CheckBox_ThreeState		: XML$ + "#PB_CheckBox_ThreeState | "		: EndIf
				If Right(XML$,3)=" | " : XML$ = Left(XML$, Len(XML$)-3) : EndIf
				XML$ + "'"
				
			EndIf
			
			XML$ + "/>"
			
			XML$ = Align(XML$, Align, Margin$, ColSpan, RowSpan)
			
			XMLBuffer$		+ Indent() + XML$ + CR$
			LastXMLLine$	= XML$ + CR$			; Store last XML-Line, to show in Error-Message
			
			ProcedureReturn Indent() + XML$
			
		EndProcedure
		Procedure.s		ComboBox(ID=#PB_Ignore, Name$="", Flags=0, Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
			
			Protected XML$ = "<ComboBox"
			
			Auswertung_Parameter_ID																				; Parameter: ID
			Auswertung_Parameter_Name																			; Parameter: Name$
			Auswertung_Parameter_Width																			; Parameter: Width
			Auswertung_Parameter_Height																		; Parameter: Height
			Auswertung_Parameter_XmlParam																		; Parameter: XmlParam$
			
			If Flags																									; Parameter: Flags
				
				XML$ + " flags='"
				
				If flags & #PB_ComboBox_Editable		= #PB_ComboBox_Editable		: XML$ + "#PB_ComboBox_Editable | "		: EndIf
				If flags & #PB_ComboBox_LowerCase	= #PB_ComboBox_LowerCase	: XML$ + "#PB_Canvas_ClipMouse | "	: EndIf
				If flags & #PB_ComboBox_UpperCase	= #PB_ComboBox_UpperCase	: XML$ + "#PB_Canvas_Keyboard | "	: EndIf
				If flags & #PB_ComboBox_Image			= #PB_ComboBox_Image			: XML$ + "#PB_Canvas_DrawFocus | "	: EndIf
				If Right(XML$,3)=" | " : XML$ = Left(XML$, Len(XML$)-3) : EndIf
				XML$ + "'"
				
			EndIf
			
			XML$ + "/>"
			
			XML$ = Align(XML$, Align, Margin$, ColSpan, RowSpan)
			
			XMLBuffer$		+ Indent() + XML$ + CR$
			LastXMLLine$	= XML$ + CR$			; Store last XML-Line, to show in Error-Message
			
			ProcedureReturn Indent() + XML$
			
		EndProcedure
		
		Procedure.s		Image(ID=#PB_Ignore, Name$="", ImageID=#PB_Ignore , Flags=0, Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
			
			Protected XML$ = "<image"
			
			Auswertung_Parameter_ID																				; Parameter: ID
			Auswertung_Parameter_Name																			; Parameter: Name$
			Auswertung_Parameter_ImageID																		; Parameter: ImageID
			Auswertung_Parameter_Width																			; Parameter: Width
			Auswertung_Parameter_Height																		; Parameter: Height
			Auswertung_Parameter_XmlParam																		; Parameter: XmlParam$
			
			If Flags																									; Parameter: Flags
				
				XML$ + " flags='"
				
				If flags & #PB_Image_Border	= #PB_Image_Border	: XML$ + "#PB_Image_Border | "		: EndIf
				If flags & #PB_Image_Raised	= #PB_Image_Raised	: XML$ + "#PB_Image_Raised | "		: EndIf
				If Right(XML$,3)=" | " : XML$ = Left(XML$, Len(XML$)-3) : EndIf
				XML$ + "'"
				
			EndIf
			
			XML$ + "/>"
			
			XML$ = Align(XML$, Align, Margin$, ColSpan, RowSpan)
			
			XMLBuffer$		+ Indent() + XML$ + CR$
			LastXMLLine$	= XML$ + CR$			; Store last XML-Line, to show in Error-Message
			
			ProcedureReturn Indent() + XML$
			
		EndProcedure
		
		Procedure.s		Text(ID=#PB_Ignore, Name$="", Text$="", Flags=0, Width=#PB_Default, Height=#PB_Default, Align=#PB_Default, Margin$="", ColSpan=#PB_Ignore, RowSpan=#PB_Ignore, XmlParam$="")
			
			Protected XML$ = "<Text"
			
			Auswertung_Parameter_ID																				; Parameter: ID
			Auswertung_Parameter_Name																			; Parameter: Name$
			Auswertung_Parameter_Text																			; Parameter: Text$
			Auswertung_Parameter_Width																			; Parameter: Width
			Auswertung_Parameter_Height																		; Parameter: Height
			Auswertung_Parameter_XmlParam																		; Parameter: XmlParam$
			
			If Flags																									; Parameter: Flags
				
				XML$ + " flags='"
				
				If flags & #PB_Text_Center		: XML$ + "#PB_Text_Center | "		: EndIf
				If flags & #PB_Text_Right		: XML$ + "#PB_Text_Right | "		: EndIf
				If flags & #PB_Text_Border		: XML$ + "#PB_Text_Border | "		: EndIf
				If Right(XML$,3)=" | " : XML$ = Left(XML$, Len(XML$)-3) : EndIf
				XML$ + "'"
				
			EndIf
			
			XML$ + "/>"
			
			If Len(Margin$)
				XML$ = Align(XML$, Align, Margin$, ColSpan, RowSpan, #True)
			Else
				XML$ = Align(XML$, Align, Margin$, ColSpan, RowSpan)
			EndIf
			
			XMLBuffer$		+ Indent() + XML$ + CR$
			LastXMLLine$	= XML$ + CR$			; Store last XML-Line, to show in Error-Message
			
			ProcedureReturn Indent() + XML$
			
		EndProcedure
		;<
	
EndModule

; IDE Options = PureBasic 5.42 Beta 3 LTS (Windows - x86)
; CursorPosition = 152
; FirstLine = 50
; Folding = yvjAAIAAACIAEu
; EnableUnicode
; EnableXP
; EnableCompileCount = 56
; EnableBuildCount = 0
; EnableExeConstant